import java.util.Arrays;
import java.util.List;

import org.virtualbox_4_3.CleanupMode;
import org.virtualbox_4_3.IEvent;
import org.virtualbox_4_3.IEventListener;
import org.virtualbox_4_3.IMachine;
import org.virtualbox_4_3.IMedium;
import org.virtualbox_4_3.IProgress;
import org.virtualbox_4_3.ISession;
import org.virtualbox_4_3.ISnapshotTakenEvent;
import org.virtualbox_4_3.IVirtualBox;
import org.virtualbox_4_3.LockType;
import org.virtualbox_4_3.VBoxEventType;
import org.virtualbox_4_3.VirtualBoxManager;

public class VBDebug_4_3 {
   
   public static void main(String[] args) {
      VirtualBoxManager vboxManager = VirtualBoxManager.createInstance(null);
      vboxManager.connect("http://localhost:18083", "", "");
      IVirtualBox vbox = vboxManager.getVBox();
      System.out.println("VBox Version : " + vbox.getVersion() + " r" + vbox.getRevision());
      
      IMachine newVm = vbox.createMachine(null, "Test", null, "Other", null);
      newVm.saveSettings();
      vbox.registerMachine(newVm);
      
      IMachine vm = vbox.findMachine("Test");
      ISession session = vboxManager.getSessionObject();
      vm.lockMachine(session, LockType.Shared);
      
      IEventListener listener = vbox.getEventSource().createListener();
      vbox.getEventSource().registerListener(listener, Arrays.asList(VBoxEventType.Any), false);
      
      IProgress progress = session.getConsole().takeSnapshot("test", "test");
      progress.waitForCompletion(-1);
      session.unlockMachine();
      
      boolean running = true;
      while (running) {
         IEvent rawEvent = vbox.getEventSource().getEvent(listener, 0);
         if (rawEvent != null) {
            if (rawEvent.getType().equals(VBoxEventType.OnSnapshotTaken)) {
               try {
                  ISnapshotTakenEvent event = ISnapshotTakenEvent.queryInterface(rawEvent);
                  String vmUuid = event.getMachineId();
                  String snapUuid = event.getSnapshotId();
                  System.out.println("Snapshot UUID " + snapUuid + " was taken on " + vmUuid);
               } catch (Throwable t) {
                  t.printStackTrace();
               }
            } else {
               System.out.println("Skipping " + rawEvent.getType());
            }
         } else {
            running = false;
         }
      }
      
      List<IMedium> mediums = vm.unregister(CleanupMode.DetachAllReturnHardDisksOnly);
      progress = vm.deleteConfig(mediums);
      progress.waitForCompletion(-1);
      if (progress.getResultCode() == 0) {
         System.out.println("Deleted VM");
      }
      
      vbox.getEventSource().unregisterListener(listener);
      vboxManager.cleanup();
   }
   
}
